@@ -1,6 +1,6 @@
 import logging
 import random
-from typing import Tuple, Any
+from typing import Tuple, Any, Optional, List, Dict
 
 import numpy as np
 import pandas as pd
@@ -18,13 +18,23 @@ log = logging.getLogger(__name__)
 
 class PointingSeries(Module, IAutonomous):
     """Module for running pointing series."""
-    __module__ = 'pyobs.modules.robotic'
 
-    def __init__(self, alt_range: Tuple[float, float] = (30., 85.), num_alt: int = 8,
-                 az_range: Tuple[float, float] = (0., 360.), num_az: int = 24,
-                 dec_range: Tuple[float, float] = (-80., 80.), min_moon_dist: float = 15., finish: int = 90,
-                 exp_time: float = 1., acquisition: str = 'acquisition', telescope: str = 'telescope',
-                 **kwargs: Any):
+    __module__ = "pyobs.modules.robotic"
+
+    def __init__(
+        self,
+        alt_range: Tuple[float, float] = (30.0, 85.0),
+        num_alt: int = 8,
+        az_range: Tuple[float, float] = (0.0, 360.0),
+        num_az: int = 24,
+        dec_range: Tuple[float, float] = (-80.0, 80.0),
+        min_moon_dist: float = 15.0,
+        finish: int = 90,
+        exp_time: float = 1.0,
+        acquisition: str = "acquisition",
+        telescope: str = "telescope",
+        **kwargs: Any,
+    ):
         """Initialize a new auto focus system.
 
         Args:
@@ -48,23 +58,23 @@ class PointingSeries(Module, IAutonomous):
         self._num_az = num_az
         self._dec_range = dec_range
         self._min_moon_dist = min_moon_dist
-        self._finish = 1. - finish / 100.
+        self._finish = 1.0 - finish / 100.0
         self._exp_time = exp_time
         self._acquisition = acquisition
         self._telescope = telescope
 
         # if Az range is [0, 360], we got north double, so remove one step
-        if self._az_range == (0., 360.):
-            self._az_range = (0., 360. - 360. / self._num_az)
+        if self._az_range == (0.0, 360.0):
+            self._az_range = (0.0, 360.0 - 360.0 / self._num_az)
 
         # add thread func
         self.add_background_task(self._run_thread, False)
 
-    async def start(self, **kwargs: Any):
+    async def start(self, **kwargs: Any) -> None:
         """Starts a service."""
         pass
 
-    async def stop(self, **kwargs: Any):
+    async def stop(self, **kwargs: Any) -> None:
         """Stops a service."""
         pass
 
@@ -72,32 +82,36 @@ class PointingSeries(Module, IAutonomous):
         """Whether a service is running."""
         return True
 
-    async def _run_thread(self):
+    async def _run_thread(self) -> None:
         """Run a pointing series."""
 
         # create grid
-        grid = {'alt': [], 'az': [], 'done': []}
+        grid: Dict[str, List[Any]] = {"alt": [], "az": [], "done": []}
         for az in np.linspace(self._az_range[0], self._az_range[1], self._num_az):
             for alt in np.linspace(self._alt_range[0], self._alt_range[1], self._num_alt):
-                grid['alt'] += [alt]
-                grid['az'] += [az]
-                grid['done'] += [False]
+                grid["alt"] += [alt]
+                grid["az"] += [az]
+                grid["done"] += [False]
 
         # to dataframe
-        grid = pd.DataFrame(grid).set_index(['alt', 'az'])
+        pd_grid = pd.DataFrame(grid).set_index(["alt", "az"])
 
         # get acquisition and telescope units
         acquisition = await self.proxy(self._acquisition, IAcquisition)
         telescope = await self.proxy(self._telescope, ITelescope)
 
+        # check observer
+        if self.observer is None:
+            raise ValueError("No observer given.")
+
         # loop until finished
         while True:
             # get all entries without offset measurements
-            todo = list(grid[~grid['done']].index)
-            if len(todo) / len(grid) < self._finish:
-                log.info('Finished.')
+            todo = list(pd_grid[~pd_grid["done"]].index)
+            if len(todo) / len(pd_grid) < self._finish:
+                log.info("Finished.")
                 break
-            log.info('Grid points left to do: %d', len(todo))
+            log.info("Grid points left to do: %d", len(todo))
 
             # get moon
             moon = self.observer.moon_altaz(Time.now())
@@ -107,8 +121,9 @@ class PointingSeries(Module, IAutonomous):
                 # pick a random index and remove from list
                 alt, az = random.sample(todo, 1)[0]
                 todo.remove((alt, az))
-                altaz = SkyCoord(alt=alt * u.deg, az=az * u.deg, frame='altaz', obstime=Time.now(),
-                                 location=self.observer.location)
+                altaz = SkyCoord(
+                    alt=alt * u.deg, az=az * u.deg, frame="altaz", obstime=Time.now(), location=self.observer.location
+                )
 
                 # get RA/Dec
                 radec = altaz.icrs
@@ -123,12 +138,12 @@ class PointingSeries(Module, IAutonomous):
                 # to do list empty?
                 if len(todo) == 0:
                     # could not find a grid point
-                    log.info('Could not find a suitable grid point, resetting todo list for next entry...')
-                    todo = list(grid.index)
+                    log.info("Could not find a suitable grid point, resetting todo list for next entry...")
+                    todo = list(pd_grid.index)
                     continue
 
             # log finding
-            log.info('Picked grid point at Alt=%.2f, Az=%.2f (%s).', alt, az, radec.to_string('hmsdms'))
+            log.info("Picked grid point at Alt=%.2f, Az=%.2f (%s).", alt, az, radec.to_string("hmsdms"))
 
             # acquire target and process result
             try:
@@ -140,20 +155,30 @@ class PointingSeries(Module, IAutonomous):
 
                 #  process result
                 if acq is not None:
-                    self._process_acquisition(**acq)
+                    await self._process_acquisition(**acq)
 
             except (ValueError, InvocationException):
-                log.info('Could not acquire target.')
+                log.info("Could not acquire target.")
                 continue
 
             # finished
-            grid.loc[alt, az] = True
+            pd_grid.loc[alt, az] = True
 
         # finished
-        log.info('Pointing series finished.')
+        log.info("Pointing series finished.")
 
-    def _process_acquisition(self, datetime: str, ra: float, dec: float, alt: float, az: float,
-                             off_ra: float = None, off_dec: float = None, off_alt: float = None, off_az: float = None):
+    async def _process_acquisition(
+        self,
+        datetime: str,
+        ra: float,
+        dec: float,
+        alt: float,
+        az: float,
+        off_ra: Optional[float] = None,
+        off_dec: Optional[float] = None,
+        off_alt: Optional[float] = None,
+        off_az: Optional[float] = None,
+    ) -> None:
         """Process the result of the acquisition. Either ra_off/dec_off or alt_off/az_off must be given.
 
         Args:
@@ -170,4 +195,4 @@ class PointingSeries(Module, IAutonomous):
         pass
 
 
-__all__ = ['PointingSeries']
+__all__ = ["PointingSeries"]
