@@ -1,56 +1,58 @@
-import threading
+from abc import ABCMeta, abstractmethod
 from typing import Dict, Optional, Any, Type, List
+from astroplan import ObservingBlock
 
-from astroplan import Observer, ObservingBlock
-
-from pyobs.comm import Comm
 from pyobs.utils.time import Time
-from pyobs.vfs import VirtualFileSystem
 from .task import Task
+from ..object import Object
 
 
-class TaskArchive(object):
-    def __init__(self, comm: Optional[Comm] = None, vfs: Optional[VirtualFileSystem] = None,
-                 observer: Optional[Observer] = None, **kwargs: Any):
-        self.comm = comm
-        self.vfs = vfs
-        self.observer = observer
+class TaskArchive(Object, metaclass=ABCMeta):
+    def __init__(self, **kwargs: Any):
+        Object.__init__(self, **kwargs)
 
-    def open(self) -> None:
+    async def open(self) -> None:
         pass
 
-    def close(self) -> None:
+    async def close(self) -> None:
         pass
 
     def _create_task(self, klass: Type[Task], **kwargs: Any) -> Task:
-        return klass(**kwargs, tasks=self, comm=self.comm, vfs=self.vfs, observer=self.observer)
+        return self.get_object(klass, tasks=self, **kwargs)
 
-    def last_changed(self) -> Optional[Time]:
+    @abstractmethod
+    async def last_changed(self) -> Optional[Time]:
         """Returns time when last time any blocks changed."""
-        raise NotImplementedError
+        ...
 
-    def last_scheduled(self) -> Optional[Time]:
+    @abstractmethod
+    async def last_scheduled(self) -> Optional[Time]:
         """Returns time of last scheduler run."""
-        raise NotImplementedError
+        ...
 
-    def get_schedulable_blocks(self) -> List[ObservingBlock]:
+    @abstractmethod
+    async def get_schedulable_blocks(self) -> List[ObservingBlock]:
         """Returns list of schedulable blocks.
 
         Returns:
             List of schedulable blocks
         """
-        raise NotImplementedError
+        ...
 
-    def update_schedule(self, blocks: List[Task], start_time: Time) -> None:
+    @abstractmethod
+    async def update_schedule(self, blocks: List[Task], start_time: Time) -> None:
         """Update the list of scheduled blocks.
 
         Args:
             blocks: Scheduled blocks.
             start_time: Start time for schedule.
         """
-        raise NotImplementedError
+        ...
 
-    def get_pending_tasks(self, start_before: Time, end_after: Time, include_running: bool = True) -> Dict[str, Task]:
+    @abstractmethod
+    async def get_pending_tasks(
+        self, start_before: Time, end_after: Time, include_running: bool = True
+    ) -> Dict[str, Task]:
         """Fetch pending tasks from portal.
 
         Args:
@@ -65,8 +67,9 @@ class TaskArchive(object):
             Timeout: If request timed out.
             ValueError: If something goes wrong.
         """
-        raise NotImplementedError
+        ...
 
+    @abstractmethod
     def get_task(self, time: Time) -> Optional[Task]:
         """Returns the active task at the given time.
 
@@ -76,19 +79,19 @@ class TaskArchive(object):
         Returns:
             Task at the given time.
         """
-        raise NotImplementedError
+        ...
 
-    def run_task(self, task: Task, abort_event: threading.Event) -> bool:
+    @abstractmethod
+    async def run_task(self, task: Task) -> bool:
         """Run a task.
 
         Args:
             task: Task to run
-            abort_event: Abort event
 
         Returns:
             Success or not
         """
-        raise NotImplementedError
+        ...
 
 
-__all__ = ['TaskArchive']
+__all__ = ["TaskArchive"]
